{ Version history:
  1.0.0.0 - 12/12/01
            Inital version
  1.0.0.1 - 12/13/01
            Added IB Dialect.  Changed dialog size for XP.
  1.0.0.2 - 12/14/01
            Changed combo box text to "1" for default value (oops).
            Changed dialog size and controls position for XP.
}
unit fmain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, DB, DBTables;

type
  TfrmMain = class(TForm)
    btnClose: TButton;
    lblBDEPath: TLabel;
    lblBDEConfigFile: TLabel;
    edtBDEPath: TEdit;
    edtBDEConfig: TEdit;
    lblBDEVersion: TLabel;
    edtBDEVersion: TEdit;
    btnTestBDE: TButton;
    tblTemp: TTable;
    lblIBDialect: TLabel;
    cmbIBDialect: TComboBox;
    procedure btnCloseClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure btnTestBDEClick(Sender: TObject);
    procedure cmbIBDialectChange(Sender: TObject);
  private
    { Private declarations }
    function GetFileVersionStr(s : string) : string;
    function GetBDERegistryInfo : boolean;
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

uses
  Registry;

const
  EmptyStr = '';

var
  bSetIBDialectOnClose : boolean;

{$R *.dfm}

procedure TfrmMain.btnCloseClick(Sender: TObject);
var
  Reg : TRegistry;
  s   : string;
begin
  if bSetIBDialectOnClose then
  begin
    Reg := TRegistry.Create;
    try
      Reg.RootKey := HKEY_LOCAL_MACHINE;
      if Reg.OpenKey('SOFTWARE\Borland\Database Engine\Settings\Drivers\Intrbase\Db Open', true) then
      begin
        if cmbIBDialect.ItemIndex = 0 then
          s := '1'
        else
          s := '3';
        Reg.WriteString('SQLDIALECT', s);
        Reg.CloseKey;
      end;
    finally
      Reg.Free;
    end;
  end;
  Close;
end;

function TfrmMain.GetFileVersionStr(s : string) : string;
const
  InfoNum = 10;
  InfoStr: array[1..InfoNum] of string = ('CompanyName', 'FileDescription', 'FileVersion', 'InternalName', 'LegalCopyright', 'LegalTradeMarks', 'OriginalFileName', 'ProductName', 'ProductVersion', 'Comments');
var
  n, Len: DWORD;
  Buf: PChar;
  Value: PChar;
begin
  Result := EmptyStr;
  n := GetFileVersionInfoSize(PChar(S), n);
  if n > 0 then
  begin
    Buf := AllocMem(n);
    GetFileVersionInfo(PChar(S), 0, n, Buf);
    if VerQueryValue(Buf, PChar('StringFileInfo\040904E4\' + InfoStr[3]), Pointer(Value), Len) then
    Result := Value;
    FreeMem(Buf, n);
  end;
end;

function TfrmMain.GetBDERegistryInfo : boolean;
var
  Reg : TRegistry;
  s   : string;
begin
  Result := false;
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    if Reg.OpenKey('SOFTWARE\Borland\Database Engine', False) then
    begin
      s := Reg.ReadString('DLLPATH');
      if s <> EmptyStr then
      begin
        if s[Length(s)] = '\' then
          Delete(s, Length(s), 1);
        edtBDEPath.Text := s;
      end;
      s := Reg.ReadString('CONFIGFILE01');
      if s <> EmptyStr then
        edtBDEConfig.Text := s;
      Reg.CloseKey;
      Result := true;
      s := EmptyStr;
      if Reg.OpenKey('SOFTWARE\Borland\Database Engine\Settings\Drivers\Intrbase\Db Open', False) then
      begin
        s := Reg.ReadString('SQLDIALECT');
        { if we read 3 from SQL DIALECT, set the combo box to 3, otherwise default to 1}
        if s = '3' then
          cmbIBDialect.ItemIndex := 1
        else
          cmbIBDialect.ItemIndex := 0;
      end;
      Reg.CloseKey;
    end
    else
    {BDE isn't installed, so disable the IB Dialect combo box}
    begin
      cmbIBDialect.ItemIndex := 0;
      cmbIBDialect.Enabled   := false;
      cmbIBDialect.Color     := clBtnFace;
    end;
  finally
    Reg.Free;
  end;
end;

procedure TfrmMain.FormActivate(Sender: TObject);
const
  BDE_NOT_FOUND = 'BDE registry entries not found.';
var
  s       : string;
  eBDEVer : extended;
begin
  cmbIBDialect.Hint := 'Sets the registry key for the SQL Dialect for the InterBase ' +
     'driver.'#13 + 'This is disabled if BDE version 5.2 or later is not installed.';
  bSetIBDialectOnClose := false;
  if GetBDERegistryInfo then
  begin
    { note: there could be multiple paths in the BDE DLL registry key }
    {       this code assumes there isn't }
    s := GetFileVersionStr(edtBDEPath.Text + '\IDAPI32.DLL');
    if s <> EmptyStr then
    begin
      edtBDEVersion.Text := s;
      eBDEVer := 0;
      try
        eBDEVer := StrToFloat(Copy(s, 1, Pos('.', s)+1));
      except
       on E: EConvertError do
         {nothing};    { ignore conversion error exception if s wasn't a valid number }
       end;
      if eBDEVer < 5.2 then
      begin {if BDE 5.2 or later isn't installed, disable IB dialect combo box}
        cmbIBDialect.ItemIndex := 0;
        cmbIBDialect.Enabled   := false;
        cmbIBDialect.Color     := clBtnFace;
      end;
    end
    else
    begin
      edtBDEVersion.Text := edtBDEPath.Text + '\IDAPI32.DLL not found.';
      cmbIBDialect.ItemIndex := 0;
      cmbIBDialect.Enabled   := false;
      cmbIBDialect.Color     := clBtnFace;
    end;
  end
  else
  begin
    edtBDEPath.Text    := BDE_NOT_FOUND;
    edtBDEConfig.Text  := BDE_NOT_FOUND;
    edtBDEVersion.Text := BDE_NOT_FOUND;
  end;
end;

procedure TfrmMain.btnTestBDEClick(Sender: TObject);
var
  bSuccess : boolean;
begin
  bSuccess := false;
  if MessageDlg('This will attempt to create a temporary Paradox table'#13 +
                'and open it to see if the BDE is properly installed.'#13 +
                'If successful, the temporary table will be deleted.'#13  +
                'Continue?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
  begin
    tblTemp.Active := False;
    tblTemp.DatabaseName := GetEnvironmentVariable('TEMP');
    if not tblTemp.Exists then
    begin
      try
        tblTemp.CreateTable;
      except
        MessageDlg('Unable to create temporary Paradox table.', mtError, [mbOK], 0);
      end;
      if tblTemp.Exists then
      begin
        bSuccess := true;
        tblTemp.Active := true;
        tblTemp.Active := false;
        try
          tblTemp.DeleteTable;
        except
          MessageDlg('Unable to remove temporary Paradox table.'#13 +
                     tblTemp.DatabaseName + '\' + tblTemp.TableName, mtError, [mbOK], 0);
        end;
      end;
    end
    else
    begin
      { try to delete the table and then tell them to run the test again }
      if MessageDlg('Temporary Paradox table already exists.'#13 +
                 'Delete it so you can run the text again?',
                 mtConfirmation, [mbYes, mbNo], 0) = mrYes then
        try
          tblTemp.DeleteTable;
        except
          MessageDlg('Unable to remove temporary Paradox table.'#13 +
          tblTemp.DatabaseName + '\' + tblTemp.TableName, mtError, [mbOK], 0);
        end;
    end;
    if bSuccess then
      MessageDlg('The BDE is installed and working properly.', mtInformation, [mbOK], 0);
  end;
end;


procedure TfrmMain.cmbIBDialectChange(Sender: TObject);
begin
  bSetIBDialectOnClose := true;
end;

end.
